import { SerializableValue } from '@previewjs/serializable-values';
import { TypeAnalyzer, ValueType, CollectedTypes } from '@previewjs/type-analyzer';
import { Reader } from '@previewjs/vfs';
import { Logger } from 'pino';

type Analyzer = {
    typeAnalyzer: Omit<TypeAnalyzer, "dispose">;
    crawlFiles: (filePaths: string[]) => Promise<{
        components: Component[];
        stories: Story[];
    }>;
    dispose: () => void;
};
type AnalyzerFactory = (options: {
    rootDir: string;
    reader?: Reader;
    logger?: Logger;
}) => Analyzer;
interface BasePreviewable {
    id: string;
    sourcePosition: FileSourcePosition;
}
interface Component extends BasePreviewable {
    exported: boolean;
    analyze: () => Promise<ComponentAnalysis>;
}
interface ComponentAnalysis {
    props: ValueType;
    types: CollectedTypes;
}
interface Story extends BasePreviewable {
    analyze: () => Promise<StoryAnalysis>;
    associatedComponent: BasicComponent | null;
}
interface StoryAnalysis {
    args: StoryArgs | null;
}
type StoryArgs = {
    sourcePosition: FileSourcePosition;
    value: SerializableValue;
};
type FileSourcePosition = {
    start: number;
    end: number;
};
type BasicComponent = Pick<Component, "id" | "analyze">;

export type { Analyzer as A, BasePreviewable as B, Component as C, FileSourcePosition as F, Story as S, AnalyzerFactory as a, ComponentAnalysis as b, StoryAnalysis as c, StoryArgs as d, BasicComponent as e };
